/* Serial port interface                                                     */

/*--- Microsoft C 5.0 compiler include files                                 */

#include <conio.h>

/*--- other includes                                                         */

#include "driver.h"

#define  SERIAL
#include "serial.h"

#include "utility.h"

/*---------------------------------------------------------------------------*/
/* Local constants (of no global interest to anyone else)                    */

#define RATECLK  115200.0              /* serial clock for bit rates         */
#define NORMALRATE 1200.0              /* ending bit rate                    */

/*---------------------------------------------------------------------------*/
/* Initialize serial port interface to the array                             */

void SetupPort(void) {

  int   portval;                                 /* temporary port value     */
  unsigned int divisor;                          /* bit rate divisor value   */

  if (Faking)                                    /* skip if no real array    */
     return;

  PulseReset();                                  /* reset the array          */

  portval = inp(CommPort+MSR);                   /* clear DCTS bit           */

/*--- set the bit rate                                                       */

  divisor = (unsigned int) (RATECLK / (double) CommRate);

  portval = inp(CommPort+LCR);
  outp(CommPort+LCR,portval | DLAB);

  outp(CommPort+DLL,divisor & 0x00ff);
  outp(CommPort+DLH,divisor >> 8);

/*--- set transmission characteristics                                       */

  outp(CommPort+LCR,PSTK+EPS+PEN+S8);            /* parity=0, 8 data, 1 stop */

  WaitSec(0.1);

  return;

} /* SetupPort */



/*---------------------------------------------------------------------------*/
/* Reset port to something more normal as a kind gesture to next program     */

void ResetPort(void) {

  int   portval;                                 /* temporary port value     */
  unsigned int divisor;                          /* bit rate divisor value   */

  if (Faking)                                    /* skip if no real array    */
     return;

  SetReset();                                    /* force and hold reset     */

  PulseChain();                                  /* clear the chain in bit   */

/*--- set the bit rate                                                       */

  divisor = (unsigned int) (RATECLK / NORMALRATE);

  portval = inp(CommPort+LCR);
  outp(CommPort+LCR,portval | DLAB);

  outp(CommPort+DLL,divisor & 0x00ff);
  outp(CommPort+DLH,divisor >> 8);

/*--- set transmission characteristics                                       */

  outp(CommPort+LCR,0x03);                       /* 8 data, 1 stop, no pty   */

  return;

} /* ResetPort */



/*---------------------------------------------------------------------------*/
/* Send an array element address                                             */
/* Requires a "1" parity bit                                                 */
/* Must wait for chars to clear shift register before switching gears...     */

void SendAddress(int address) {

  int   tempLCR;                                 /* hold LCR value           */
  int   portval;                                 /* temporary port value     */
  int   thisLCR;

  if (Faking)                                    /* skip if no real array    */
     return;

  do {                                           /* spin if char in transit  */
     portval = inp(CommPort+LSR);
  } while ((0 == (portval & TSRE)) || (0 == (portval & THRE)));

  tempLCR = inp(CommPort+LCR);                   /* reset to "1" parity      */
  thisLCR = tempLCR & ~EPS;
  outp(CommPort+LCR,thisLCR);

  outp(CommPort+THR,address);                    /* send the address         */

  do {                                           /* spin while address going */
     portval = inp(CommPort+LSR);
  } while ((0 == (portval & TSRE)) || (0 == (portval & THRE)));

  thisLCR = tempLCR | EPS;
  outp(CommPort+LCR,thisLCR);                    /* reset to "0" parity      */

  return;

} /* SendAddress */


/*---------------------------------------------------------------------------*/
/* Send a group of bytes to the array                                        */

void SendBlock(char *blk,int blksize) {

  if (Faking)                                    /* skip if no real array    */
     return;

  for (; blksize > 0; blksize--) {
     SendData(*blk);
     blk++;
  }

  return;

} /* SendBlock */



/*---------------------------------------------------------------------------*/
/* Send a data byte                                                          */
/* Requires "0" parity bit, which was set up by the preceeding address byte  */

void SendData(int data) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return;

  do {                                           /* spin if holding char     */
     portval = inp(CommPort+LSR);
  } while (0 == (portval & THRE));

  outp(CommPort+THR,data);                       /* send the byte            */

  return;

} /* SendData */


/*---------------------------------------------------------------------------*/
/* Reads the data port value                                                 */
/* Returns -1 if no data is available                                        */

int  ReadPort(void) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return 0;

  portval = inp(CommPort+LSR);

  if (portval & DR) {                            /* if byte available        */
     portval = inp(CommPort+RB);
     portval &= 0x00ff;                          /* strip high byte          */
     return portval;
  }
  else
     return -1;                                  /* else flag lack of data   */

} /* ReadPort */


/*---------------------------------------------------------------------------*/
/* Pulse the array reset line                                                */
/* This is a long pulse to make sure everyone sees it                        */
/* Resets are -V on the RS-232 interface, inverted to +V on the boards       */

void PulseReset(void) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return;

  portval = inp(CommPort+MCR);                   /* get starting value       */

  outp(CommPort+MCR,portval & ~DTR);

  WaitSec(0.2);                                  /* actually delays a little */

  outp(CommPort+MCR,portval | DTR);

  return;

} /* PulseReset */


/*---------------------------------------------------------------------------*/
/* Set the array reset line                                                  */
/* This forces the array to calm down                                        */
/* Resets are -V on the RS-232 interface, inverted to +V on the boards       */

void SetReset(void) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return;

  portval = inp(CommPort+MCR);                   /* get starting value       */

  outp(CommPort+MCR,portval & ~DTR);

  return;

} /* SetReset */


/*---------------------------------------------------------------------------*/
/* Pulse the chain-out line to elicit a response from the array              */
/* This is a rather short pulse, which may have trouble over long cables     */
/* The interrupt is a positive pulse, which is inverted to -INT              */
/* The WaitSec(0.0) routine provide just the right touch of delay for very   */
/*  fast processors; you may be able to do without them at all...            */

void PulseChain(void) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return;

  portval = inp(CommPort+MSR);                   /* clear chain-in flag      */

  portval = inp(CommPort+MCR);                   /* get starting value       */

  outp(CommPort+MCR,portval | RTS);
#if 1
  WaitSec(0.0);                                  /* delay for just a bit     */
#endif
  outp(CommPort+MCR,portval & ~RTS);
  return;

} /* PulseChain */


/*---------------------------------------------------------------------------*/
/* Check for a chain-out pulse from the array                                */
/* The hardware notes that there was a pulse, not which way it went          */

int CheckChain(void) {

  int   portval;                                 /* temporary port value     */

  if (Faking)                                    /* skip if no real array    */
     return 0;

  portval = inp(CommPort+MSR);                   /* get value, reset status  */

  if (portval & DCTS) {
     return 0;                                   /* pulse OK                 */
  }
  else
     return 1;                                   /* no pulse                 */

} /* CheckChain */

